import logging
import pandas

class UserAwsAuth:
    def __init__(self, user_credential_file_path, sep_ = None, secret_key_id_col_name = None, secret_access_key_col_name = None):
        self.user_credential_file_path = user_credential_file_path
        credetial = self.get_credential(user_credential_file_path, secret_key_id_col_name, secret_access_key_col_name, sep_)
        if credetial is not None:
            self.secret_key_id = credetial[0]
            self.secret_access_key = credetial[1]
        else:
            self.secret_key_id = None
            self.secret_access_key = None



    def describe(self):
        print(self.secret_key_id, self.secret_access_key)


    def get_credential(self, user_credential_file_path, secret_key_id_col_name, secret_access_key_col_name, sep_):
        if secret_key_id_col_name is None:
            secret_key_id_col_name = "Access key ID"
        if secret_access_key_col_name is None:
            secret_access_key_col_name = "Secret access key"
        if sep_ is None:
            sep_ = ','
        try:
            df_credential = pandas.read_csv(user_credential_file_path, sep=sep_)
            if df_credential.shape[0]==0:
                logging.error("le fichier creatial spécifié est vide!")
                return None
            secret_key_id = df_credential[secret_key_id_col_name][0]
            secret_access_key = df_credential[secret_access_key_col_name][0]
            return secret_key_id, secret_access_key
        except Exception as exp:
            logging.error(f"get credential failed! {exp}")
            return None




import boto3, uuid, sys, time
from botocore.exceptions import ClientError
import logging


def generate_job_id():
    return uuid.uuid4().__str__()

def delete_all_ressources(stack_name):
    clf_client = boto3.client('cloudformation')
    clf_client.delete_stack(StackName=stack_name)

def upload_file_to_s3(s3_client, local_path, bucket, s3_key):
    s3_client.upload_file(local_path, bucket, s3_key)

def create_train_clf_stack(clf_client,
                     stack_name: str,
                     template_url: str,
                     list_parameters: list
                     ):
    try:
        clf_response = clf_client.create_stack(
            StackName = stack_name,
            TemplateURL = template_url,
            Parameters = list_parameters,
            Capabilities=['CAPABILITY_NAMED_IAM']
        )
        return clf_response['StackId']
    except ClientError as cle:
        logging.error(f"creation de la stack echouée... {cle}")
        return None

def run_ec2_command(instance_id, commands, aws_access_key_id, aws_secret_access_key, region):
    ssm_client = boto3.client('ssm', aws_access_key_id = aws_access_key_id, aws_secret_access_key = aws_secret_access_key, region_name = region)
    try:
        response = ssm_client.send_command(
                DocumentName="AWS-RunShellScript",
                Parameters={'commands': commands},
                InstanceIds=[
                    instance_id
                ]
            )
        return response
    except ClientError as cle:
        logging.error(f"impossible d'executer la commande : {commands}")
        raise Exception(cle)


def get_command_status(instance_id, command_id, aws_access_key_id, aws_secret_access_key, region):
    ssm_client = boto3.client('ssm', aws_access_key_id = aws_access_key_id, aws_secret_access_key = aws_secret_access_key, region_name = region)
    try:
        command_status = ssm_client.get_command_invocation(CommandId=command_id, InstanceId=instance_id)
        return {
            "status": command_status['Status'],
            "status_details": command_status['StatusDetails']
        }
    except ClientError as cle:
        logging.error(f"impossible d'obtenir le statut de la commande suivante: {command_id}")
        raise Exception(cle)


"""def get_ec2_instance_id(job_id, access_key_id, secret_access_key, region):
    ec2_client = boto3.client('ec2', aws_access_key_id = access_key_id, aws_secret_access_key = secret_access_key, region_name = region)
    instances = ec2_client.describe_instances(
        Filters=[
            {
                'Name': "tag:job_id",
                'Values': [
                    job_id,
                ]
            }
        ]
    )
    if len(instances['Reservations'])==0:
        raise Exception(f"impossible d'obtenir l'id de l'instance d'entrainement pour le job: {job_id}!")
    return instances['Reservations'][0]['Instances'][0]['InstanceId']
"""

def get_train_ec2_instance_status(instance_id, access_key_id, secret_access_key, region):
    ec2_client = boto3.client('ec2', aws_access_key_id=access_key_id, aws_secret_access_key=secret_access_key,
                                  region_name=region)
    ec2_response = ec2_client.describe_instance_status(
        InstanceIds = [
            instance_id,
        ],
        IncludeAllInstances=True
    )
    instances_status = ec2_response['InstanceStatuses']
    if len(instances_status)==0:
        raise Exception(f"Auncune instance avec l'identifiant suivant: {instance_id}")

    if instances_status[0]["InstanceState"]["Name"]=="pending":
        return instances_status[0]["InstanceState"]["Name"]
    if instances_status[0]["InstanceState"]["Name"] == "running":
        if instances_status[0]['InstanceStatus']['Details'][0]['Status']=='passed':
            return instances_status[0]["InstanceState"]["Name"]
        if instances_status[0]['InstanceStatus']['Details'][0]['Status']=='initializing':
            return "pending"
    return 'Failed'


def terminate_instance(instance_id, access_key_id, secret_access_key, region):
    try:
        ec2_client = boto3.client('ec2', aws_access_key_id=access_key_id, aws_secret_access_key=secret_access_key,
                                      region_name=region)
        client_response = ec2_client.terminate_instances(
            InstanceIds=[
                instance_id
            ]
        )
        return client_response
    except ClientError as cle:
        logging.error(f"impossible d'arreter l'instance d'entrainement! : {instance_id}")
        raise Exception(cle)


def delete_clf_stack(stack_id, access_key_id, secret_access_key, region):
    try:
        clf_client = boto3.client('cloudformation', aws_access_key_id=access_key_id, aws_secret_access_key=secret_access_key,
                                  region_name=region)
        clf_response = clf_client.delete_stack(
            StackName = stack_id
        )
        return clf_response
    except ClientError as cle:
        logging.error(f"impossible de supprimer la pile CloudFormation d'entrainement! : {stack_id}")
        raise Exception(cle)


def progress_bar(i):
    sys.stdout.write("\r|%s>" % ('='*i))
    sys.stdout.flush()









class TrainNomenclature:
    def __init__(self, job_id, bucket, region):
        self.job_id = job_id
        self.bucket = bucket
        self.region = region

    def get_s3_lbd_location(self):
        return f"domaine=repository/table=lambda-train/jobid={self.job_id}/lambda_train.zip"

    def get_s3_train_script_location(self):
        return f"domaine=repository/table=train-script/jobid={self.job_id}/train_script.py"

    def get_s3_requirements_location(self):
        return f"domaine=repository/table=requirements/jobid={self.job_id}/requirements.txt"

    def get_s3_stack_template_location(self):
        s3_key = f"domaine=repository/table=template_stack/jobid={self.job_id}/template_train_stack.json"
        s3_url = f"https://{self.bucket}.s3.{self.region}.amazonaws.com/{s3_key}"
        return {
            "s3_key": s3_key,
            "s3_url": s3_url
        }

    def get_train_stack_name(self):
        return f"train-stack-{self.job_id}"

    def get_lbd_train_name(self):
        return f"lbd-train-model-{self.job_id}"

    def get_role_train_name(self):
        return f"Train-Role-{self.job_id}"

    def get_profil_iam_name(self):
        return f"Train-profil-iam-{self.job_id}"




from UserAwsAuth import *
from Train import *
from Deploy import *
from deployment_helper import get_api_endpoint

if __name__ == "__main__":
    #######A l'utilisateur de les declarer########
    bucket = "pa-2022"
    device_size = "2"
    region = "eu-west-3"
    instance_type = "t2.micro"
    input_model_type = "int"
    model_s3_key = "...."
    #############################################
    local_rep = "ressources"
    template_stack_local_path = f"{local_rep}/clf_train_stack.json"
    train_script_local_path = f"{local_rep}/train_script.py"
    requirements_local_path = f"{local_rep}/requirements.txt"
    train_lbd_local_path = f"{local_rep}/lambda_train.zip"

    ami = "ami-021d41cbdefc0c994"


    credential_file_path = "C:/Users/lounhadja/PycharmProjects/PA_TDV/new_user_credentials.csv"
    auth_object = UserAwsAuth(credential_file_path)
    auth_object.describe()


    ##################################################################################################
    ################################### Partie entrainement############################################
    ##################################################################################################
    train_object = Train(bucket, auth_object, template_stack_local_path, train_script_local_path,
                         requirements_local_path, train_lbd_local_path, instance_type, ami,
                         device_size, region=region)


    prep_env_response = train_object.prepare_env()

    print("[Train] creation stack...")
    create_stack_status = train_object.create_clf_stack(prep_env_response)

    print("\n[Train] lancement ec2...")
    instance_id = train_object.lunch_train_ec2()

    print(instance_id)
    print("\n[Train] installation requerements...")
    install_req_status = train_object.install_requerments(instance_id)


    install_req_status["status"] = "Success"
    if install_req_status["status"]=="Success":
        print("\n[Train] entrainement model...")
        train_status = train_object.lunch_train_script(instance_id)
        if train_status["status"] == "Success":
            print("\n========>train finished!")

    train_object.delete_resources(instance_id)

    ##################################################################################################
    ###################################Partie Deploiement############################################
    ##################################################################################################
    template_deployment_stack_local_path = f"{local_rep}/clf_deployment_stack.json"
    deployment_lbd_local_path = f"{local_rep}/lambda_deployment.zip"

    DeployObject = Deploy(bucket, model_s3_key, input_model_type, auth_object,
                          template_deployment_stack_local_path, deployment_lbd_local_path ,
                          region=region)
    print("[Deployment] prepare_env...")
    deployment_prepare_env = DeployObject.prepare_deployment()

    print("[Deployment] deploy...")
    api_url = DeployObject.deploy(deployment_prepare_env)

    print(api_url)


from botocore.exceptions import ClientError
from easyTDV.TrainNomenclature import *
from easyTDV.train_helper import generate_job_id, upload_file_to_s3, create_train_clf_stack, progress_bar, get_train_ec2_instance_status, run_ec2_command, get_command_status, terminate_instance, delete_clf_stack
from pathlib import Path
import boto3, logging, time, json


class Train:
    dir = Path(__file__).parent
    template_stack_local_path = (dir / "resources/clf_train_stack.json").__str__()
    train_lbd_local_path = (dir / "resources/lambda_train.zip").__str__()
    def __init__(self,
                     bucket,
                     auth_object,
                     train_script_local_path,
                     requirements_local_path,
                     instance_type,
                     device_size,
                     ami = "ami-021d41cbdefc0c994",
                     region = "eu-west-3"
                     ):
        self.bucket = bucket
        self.train_script_local_path = train_script_local_path
        self.requirements_local_path = requirements_local_path
        self.instance_type = instance_type
        self.ami = ami
        self.device_size = device_size
        self.region = region
        self.access_key_id = auth_object.secret_key_id
        self.secret_access_key = auth_object.secret_access_key
        self.job_id = generate_job_id()
        self.nomenclature_object = TrainNomenclature(self.job_id, self.bucket, self.region)


    def prepare_env(self):
        """
        charger les fichiers: train.py, requirements.txt, stack_template.json et lbd_train.py vers S3
        :return: les keys S3 des fichiers chargés vers S3
        """
        s3_client = boto3.client('s3', aws_access_key_id=self.access_key_id,
                                 aws_secret_access_key=self.secret_access_key, region_name=self.region)
        s3_lbd_location = self.nomenclature_object.get_s3_lbd_location()
        s3_train_script_location = self.nomenclature_object.get_s3_train_script_location()
        s3_requirements_location = self.nomenclature_object.get_s3_requirements_location()
        s3_stack_template_location = self.nomenclature_object.get_s3_stack_template_location()
        try:
            upload_file_to_s3(s3_client, self.train_lbd_local_path, self.bucket, s3_lbd_location)
            upload_file_to_s3(s3_client, self.train_script_local_path, self.bucket, s3_train_script_location)
            upload_file_to_s3(s3_client, self.requirements_local_path, self.bucket, s3_requirements_location)
            upload_file_to_s3(s3_client, self.template_stack_local_path, self.bucket, s3_stack_template_location["s3_key"])
        except ClientError as cle:
            logging.error("[TRAIN]:preparation d'enveronement echouée...")
            raise Exception(cle)
        return {
            "s3_lbd_key": s3_lbd_location,
            "s3_train_script_key": s3_train_script_location,
            "s3_requirements_key": s3_requirements_location,
            "url_s3_stack_template": s3_stack_template_location["s3_url"]
        }


    def create_clf_stack(self, prepare_env_response, invoke_mode=0):
        """
        :param prepare_env_response: l'objet retourné par prepare_env()
        :return:
        """
        if invoke_mode not in [0, 1]:
            raise Exception("valeurs acceptées pour invoke_mode: [0:synchrone, 1: asynchrone]")
        s3_lbd_key = prepare_env_response["s3_lbd_key"]
        s3_train_script_key = prepare_env_response["s3_train_script_key"]
        s3_requirements_key = prepare_env_response["s3_requirements_key"]
        url_s3_stack_template = prepare_env_response["url_s3_stack_template"]

        stack_name = self.nomenclature_object.get_train_stack_name()
        lbd_train_name = self.nomenclature_object.get_lbd_train_name()
        role_train_name = self.nomenclature_object.get_role_train_name()
        profil_iam_name = self.nomenclature_object.get_profil_iam_name()

        list_parameters = [
            {
                'ParameterKey': 'JobID',
                'ParameterValue': self.job_id
            },
            {
                'ParameterKey': 'LbdTrainNameParameter',
                'ParameterValue': lbd_train_name
            },
            {
                'ParameterKey': 'InstanceProfilNameParameter',
                'ParameterValue': profil_iam_name
            },
            {
                'ParameterKey': 'RoleTrainNameParameter',
                'ParameterValue': role_train_name
            },
            {
                'ParameterKey': 'S3BucketParameter',
                'ParameterValue': self.bucket
            },
            {
                'ParameterKey': 'S3KeyParameter',
                'ParameterValue': s3_lbd_key
            },
            {
                'ParameterKey': 'InstanceTypeParameter',
                'ParameterValue': self.instance_type
            },
            {
                'ParameterKey': 'AmiParameter',
                'ParameterValue': self.ami
            },
            {
                'ParameterKey': 'regionParameter',
                'ParameterValue': self.region
            },
            {
                'ParameterKey': 'TrainScriptKeyParameter',
                'ParameterValue': s3_train_script_key
            },
            {
                'ParameterKey': 'RequirementScriptKeyParameter',
                'ParameterValue': s3_requirements_key
            },
            {
                'ParameterKey': 'DeviceSizeParameter',
                'ParameterValue': self.device_size
            }
        ]
        try:
            clf_client = boto3.client('cloudformation', aws_access_key_id=self.access_key_id,
                                      aws_secret_access_key=self.secret_access_key, region_name=self.region)
            stack_id = create_train_clf_stack(clf_client, stack_name, url_s3_stack_template, list_parameters)
            if invoke_mode==0:
                quit = True
                iter = 1
                while(quit):
                    time.sleep(2)
                    clf_stack_status = self.get_clf_stack_status(stack_name)
                    if clf_stack_status == 'CREATE_COMPLETE':
                        logging.info(f"[TRAIN]:statut creation CloudFormation: {clf_stack_status}")
                        return clf_stack_status
                    elif clf_stack_status == 'CREATE_IN_PROGRESS':
                        progress_bar(iter)
                        iter+=1
                    else:
                        logging.error(f"[TRAIN]:statut creation CloudFormation: {clf_stack_status}")
                        raise Exception("[TRAIN]:Creation stack CloudFormation echoué")
            if invoke_mode==1:
                create_ressources_response = {}
                create_ressources_response['StackId'] = stack_id
                create_ressources_response['StackName'] = stack_name
                return create_ressources_response
        except ClientError as cle:
            logging.error(f"[TRAIN]:creation ressources echouée!")
            raise Exception(cle)


    def get_clf_stack_status(self, stack_name):
        """
        :param stack_name: le nom de la stack CloudFormation
        :return: le status en cours
        """
        clf_client = boto3.client('cloudformation', aws_access_key_id=self.access_key_id,
                                  aws_secret_access_key=self.secret_access_key, region_name=self.region)

        try:
            clf_response = clf_client.describe_stacks(
                StackName=stack_name
            )
            return clf_response['Stacks'][0]['StackStatus']
        except ClientError as cle:
            logging.error(f"[TRAIN]:impossible d'obtenir le statut pour la stack: {stack_name}")
            raise Exception(cle)


    def lunch_train_ec2(self, invoke_mode=0):
        if invoke_mode not in [0, 1]:
            raise Exception("valeurs acceptées pour invoke_mode: [0:synchrone, 1: asynchrone]")

        lbd_client = boto3.client('lambda', aws_access_key_id=self.access_key_id,
                                  aws_secret_access_key=self.secret_access_key, region_name=self.region)
        lbd_train_name = self.nomenclature_object.get_lbd_train_name()
        try:
            invoke_lbd_response = lbd_client.invoke(
                FunctionName=lbd_train_name,
                InvocationType='RequestResponse'
            )
            instance_id = json.loads(invoke_lbd_response['Payload'].read())["instance_id"]
            if invoke_mode == 1:
                return instance_id
            if invoke_mode == 0:
                iter=1
                while(True):
                    time.sleep(2)
                    instance_status = get_train_ec2_instance_status(instance_id, self.access_key_id, self.secret_access_key, self.region)
                    if instance_status == 'running':
                        return instance_id
                    elif instance_status == 'pending':
                        progress_bar(iter)
                        iter+=1
                    else:
                        raise Exception(f"status de l'instance d'entrainement en erreur : {instance_status}")
        except ClientError as cle:
            logging.error("impossible de lancer l'instance d'entrainement!")
            raise Exception(cle)


    def install_requerments(self, instance_id, invoke_mode=0):
        if invoke_mode not in [0, 1]:
            raise Exception("valeurs acceptées pour invoke_mode: [0:synchrone, 1: asynchrone]")
        requirements_file_name = "requirements.txt"
        commands = [f"pip3.8 install -r /appli/{requirements_file_name}"]
        cmd_exec_response = run_ec2_command(instance_id, commands, self.access_key_id, self.secret_access_key,
                                            self.region)
        if invoke_mode == 1:
            return cmd_exec_response
        command_id = cmd_exec_response['Command']['CommandId']
        iter = 0
        while(True):
            time.sleep(2)
            cmd_status = get_command_status(instance_id, command_id, self.access_key_id, self.secret_access_key, self.region)
            if cmd_status["status"] == "Success":
                return cmd_status
            if cmd_status["status"] in ['Pending', 'InProgress', 'Delayed']:
                progress_bar(iter)
                iter+=1
                continue
            return {
                "status" : "failed",
                "status_details" : cmd_status["status_details"]
            }

    def lunch_train_script(self, instance_id, invoke_mode=0):
        if invoke_mode not in [0, 1]:
            raise Exception("valeurs acceptées pour invoke_mode: [0:synchrone, 1: asynchrone]")

        train_script_name = "train_script.py"
        commands = [f"sudo python3.8 /appli/{train_script_name}"]
        cmd_exec_response = run_ec2_command(instance_id, commands, self.access_key_id, self.secret_access_key,
                                            self.region)

        if invoke_mode == 1:
            return cmd_exec_response
        command_id = cmd_exec_response['Command']['CommandId']
        iter = 0
        while (True):
            time.sleep(2)
            cmd_status = get_command_status(instance_id, command_id, self.access_key_id, self.secret_access_key,
                                            self.region)
            if cmd_status["status"] == "Success":
                return cmd_status
            if cmd_status["status"] in ['Pending', 'InProgress', 'Delayed']:
                progress_bar(iter)
                iter += 1
                continue
            return {
                "status": "failed",
                "status_details": cmd_status["status_details"]
            }


    def delete_resources(self, instance_id):
        stack_name = self.nomenclature_object.get_train_stack_name()
        terminate_instance(instance_id, self.access_key_id, self.secret_access_key, self.region)
        delete_clf_stack(stack_name, self.access_key_id, self.secret_access_key, self.region)










from easyTDV.UserAwsAuth import UserAwsAuth
from botocore.exceptions import ClientError
from easyTDV.DeploymentNomenclature import *
from easyTDV.deployment_helper import generate_job_id, upload_file_to_s3, create_train_clf_stack, progress_bar, get_api_endpoint
from pathlib import Path
import boto3, logging, time


class Deploy:
    dir = Path(__file__).parent
    template_stack_local_path = (dir / "resources/clf_deployment_stack.json").__str__()
    deployment_lbd_local_path = (dir / "resources/lambda_deployment.zip").__str__()
    
    def __init__(self,
                 bucket: str,
                 model_s3_key: str,
                 input_model_type,
                 auth_object: UserAwsAuth,
                 region = "eu-west-3"
                 ):
        self.bucket = bucket
        self.model_s3_key = model_s3_key
        self.input_model_type = input_model_type
        self.access_key_id = auth_object.secret_key_id
        self.secret_access_key = auth_object.secret_access_key
        self.region = region
        self.job_id = generate_job_id()
        self.nomenclature_object = DeploymentNomenclature(self.job_id, self.bucket, self.region)


    def prepare_deployment(self):
        """
        Charger vers s3 le template CloudFormation et le code la Lambda RequestProcesssor
        :return: les keys S3 des fichiers chargés vers S3
        """
        print(f"JOBID: {self.job_id}")
        s3_client = boto3.client('s3', aws_access_key_id=self.access_key_id,
                                 aws_secret_access_key=self.secret_access_key, region_name=self.region)

        s3_lbd_request_processor_location = self.nomenclature_object.get_s3_lbd_location()
        s3_stack_template_location = self.nomenclature_object.get_s3_stack_template_location()

        try:
            upload_file_to_s3(s3_client, self.deployment_lbd_local_path, self.bucket, s3_lbd_request_processor_location)
            upload_file_to_s3(s3_client, self.template_stack_local_path, self.bucket, s3_stack_template_location["s3_key"])
        except ClientError as cle:
            logging.error("[DEPLOYMENT]:preparation d'enveronement echouée...")
            raise Exception(cle)
        return {
            "s3_lbd_key" : s3_lbd_request_processor_location,
            "url_s3_stack_template": s3_stack_template_location["s3_url"]
        }


    def create_stack(self, prepare_env_response, invoke_mode=0):
        if invoke_mode not in [0, 1]:
            raise Exception("valeurs acceptées pour invoke_mode: [0:synchrone, 1: asynchrone]")

        request_processor_s3_key = prepare_env_response["s3_lbd_key"]
        url_s3_stack_template = prepare_env_response["url_s3_stack_template"]

        deployment_stack_name = self.nomenclature_object.get_deployment_stack_name()
        lbd_request_processor_name = self.nomenclature_object.get_lbd_deployment_name()
        api_name = self.nomenclature_object.get_api_name()
        deployment_role_name = self.nomenclature_object.get_role_deployment_name()

        list_parameters = [
            {
                'ParameterKey': 'JobID',
                'ParameterValue': self.job_id
            },
            {
                'ParameterKey': 'Bucket',
                'ParameterValue': self.bucket
            },
            {
                'ParameterKey': 'LbdS3keyParameter',
                'ParameterValue': request_processor_s3_key
            },
            {
                'ParameterKey': 'ModelS3KeyParameter',
                'ParameterValue': self.model_s3_key
            },
            {
                'ParameterKey': 'InputModelTypeParameter',
                'ParameterValue': self.input_model_type
            },
            {
                'ParameterKey': 'DepFunctionName',
                'ParameterValue': lbd_request_processor_name
            },
            {
                'ParameterKey': 'Region',
                'ParameterValue': self.region
            },
            {
                'ParameterKey': 'ApiDeploymentName',
                'ParameterValue': api_name
            },
            {
                'ParameterKey': 'RoleDeploymentNameParameter',
                'ParameterValue': deployment_role_name
            }
        ]
        try:
            clf_client = boto3.client('cloudformation', aws_access_key_id=self.access_key_id,
                                      aws_secret_access_key=self.secret_access_key, region_name=self.region)

            stack_id = create_train_clf_stack(clf_client, deployment_stack_name, url_s3_stack_template, list_parameters)
            if invoke_mode==1:
                create_ressources_response = {}
                create_ressources_response['StackId'] = stack_id
                create_ressources_response['StackName'] = deployment_stack_name
                return create_ressources_response
            if invoke_mode==0:
                quit = True
                iter = 1
                while(quit):
                    time.sleep(2)
                    clf_stack_status = self.get_clf_stack_status(deployment_stack_name)
                    if clf_stack_status == 'CREATE_COMPLETE':
                        logging.info(f"[DEPLOYMENT]:statut creation CloudFormation: {clf_stack_status}")
                        return clf_stack_status
                    elif clf_stack_status == 'CREATE_IN_PROGRESS':
                        progress_bar(iter)
                        iter+=1
                    else:
                        logging.error(f"[DEPLOYMENT]:statut creation CloudFormation: {clf_stack_status}")
                        raise Exception("Creation stack CloudFormation echoué")

        except ClientError as cle:
            logging.error(f"[DEPLOYMENT]:creation ressources echouée!")
            raise Exception(cle)


    def get_clf_stack_status(self, stack_name):
        """
        :param stack_name: le nom de la stack CloudFormation
        :return: le status en cours
        """
        clf_client = boto3.client('cloudformation', aws_access_key_id=self.access_key_id,
                                  aws_secret_access_key=self.secret_access_key, region_name=self.region)

        try:
            clf_response = clf_client.describe_stacks(
                StackName=stack_name
            )
            return clf_response['Stacks'][0]['StackStatus']
        except ClientError as cle:
            logging.error(f"[DEPLOYMENT]:impossible d'obtenir le statut pour la stack: {stack_name}")
            raise Exception(cle)


    def deploy(self, prepare_env_response, invoke_mode=0):
        self.create_stack(prepare_env_response, invoke_mode=invoke_mode)
        return get_api_endpoint(self.job_id, self.access_key_id, self.secret_access_key, self.region)





import uuid, sys, logging, boto3, json
from botocore.exceptions import ClientError


def generate_job_id():
    return uuid.uuid4().__str__()

def upload_file_to_s3(s3_client, local_path, bucket, s3_key):
    s3_client.upload_file(local_path, bucket, s3_key)

def create_train_clf_stack(clf_client,
                     stack_name: str,
                     template_url: str,
                     list_parameters: list
                     ):
    try:
        clf_response = clf_client.create_stack(
            StackName = stack_name,
            TemplateURL = template_url,
            Parameters = list_parameters,
            Capabilities=['CAPABILITY_NAMED_IAM']
        )
        return clf_response['StackId']
    except ClientError as cle:
        logging.error(f"creation de la stack echouée... {cle}")
        return None


def get_restapi_id(job_id, access_key_id, secret_access_key, region):
    apigetway_client = boto3.client('apigateway', aws_access_key_id=access_key_id,
                                    aws_secret_access_key=secret_access_key,
                                    region_name=region)
    try:
        rest_apis = apigetway_client.get_rest_apis(
            limit=499
        )

        list_rest_apis = list(
            filter(lambda item: ("JOBID" in item['tags']) and (item['tags']["JOBID"] == job_id), rest_apis['items']))
        if len(list_rest_apis) == 0:
            raise Exception(f"Aucune API Gateway avec le tag JOBID:<{job_id}>")
        api = list_rest_apis[0]
        return api['id']
    except ClientError as cle:
        logging.error("[DEPLOYMENT]: erreur recuperation id API GATEWAY")
        raise Exception(cle)


def get_api_endpoint(job_id, access_key_id, secret_access_key, region):
    try:
        api_id = get_restapi_id(job_id, access_key_id, secret_access_key, region)
        apigetway_client = boto3.client('apigateway', aws_access_key_id=access_key_id,
                                        aws_secret_access_key=secret_access_key,
                                        region_name=region)
        apigetway_response = apigetway_client.get_export(
                                restApiId = api_id,
                                stageName = "apiprediction",
                                exportType = 'oas30'
                            )

        url_component = json.loads(apigetway_response['body'].read())['servers']
        url = url_component[0]['url'].split('/')[:-1]
        resource_methode = "apiprediction/prediction"
        url.append(resource_methode)
        return '/'.join(url)
    except ClientError as cle:
        logging.error("[DEPLOYMENT]: get_api_endpoint()")
        raise Exception(cle)
    except Exception as exp:
        logging.error("[DEPLOYMENT]: get_api_endpoint()")
        raise Exception(exp)



def progress_bar(i):
    sys.stdout.write("\r|%s>" % ('='*i))
    sys.stdout.flush()



from setuptools import setup, find_packages, glob


VERSION = '1.1.6'
DESCRIPTION = "easyTDV: Train, Deploy and Vizualisation"
LONG_DESCRIPTION = "Framwork pour l'entrainement, deploiement et la visualisation à temps réel des modèles ML sur AWS"

# Setting up
setup(
    name="easyTDV",
    version=VERSION,
    author="HADJALI Lounas",
    author_email="<hadjalilounas@gmail.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['boto3', 'pandas'],
    keywords=['python', 'ML', 'Deployment', 'Train', 'AWS'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ], 
    package_data={'easyTDV': ['resources/clf_deployment_stack.json', 'resources/clf_train_stack.json', 'resources/lambda_deployment.zip', 'resources/lambda_train.zip']},
    include_package_data=True
)



class DeploymentNomenclature:
    def __init__(self, job_id, bucket, region):
        self.job_id = job_id
        self.bucket = bucket
        self.region = region

    def get_s3_lbd_location(self):
        return f"domaine=repository/table=lambda-deployment/jobid={self.job_id}/lambda_deployment.zip"

    def get_s3_model_location(self):
        pass

    def get_lbd_deployment_name(self):
        return f"lbd-request-processor-{self.job_id}"

    def get_api_name(self):
        return f"api-predection-{self.job_id}"

    def get_deployment_stack_name(self):
        return f"deployment-stack-{self.job_id}"

    def get_role_deployment_name(self):
        return f"Deployment-Role-{self.job_id}"

    def get_s3_stack_template_location(self):
        s3_key = f"domaine=repository/table=template_stack/jobid={self.job_id}/template_deployment_stack.json"
        s3_url = f"https://{self.bucket}.s3.{self.region}.amazonaws.com/{s3_key}"
        return {
            "s3_key": s3_key,
            "s3_url": s3_url
        }

